#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Build script for the Bos Wars engine.
#
#  (c) Copyright 2010-2013 by Francois Beerten.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# This script is compatible with both Python 2 and Python 3.

import os
import glob
import sys
try:
   import Queue as queue
except ImportError:
   import queue
import threading
from warnings import warn
from fabricate import *


target = 'boswars'
gccflags = '-Wall -fsigned-char -D_GNU_SOURCE=1 -D_REENTRANT'.split()
incpaths = 'engine/include engine/guichan/include'.split()

def find(startdir, pattern):
    import fnmatch
    results = []
    for dirpath,dirnames,files in os.walk(startdir):
       for f in files:
          # Ignore dot files.  GNU Emacs especially creates lock files
          # with names like ".#script.cpp".  Don't try to compile those.
          if fnmatch.fnmatch(f, pattern) and not f.startswith("."):
             results.append(dirpath+'/'+f)
    return results
    
sources = find('engine', '*.cpp')

class Gcc(object):
  def __init__(self, cflags=[], ldflags=[], cc='g++', builddir='fbuild', 
                      usepkgconfig=True):
     self.cflags = gccflags + list(cflags)
     self.ldflags = list(ldflags)
     self.cc = cc
     self.builddir = builddir
     self.usepkgconfig = usepkgconfig
  def copy(self):
     g = Gcc(self.cflags, self.ldflags, self.cc, self.builddir, self.usepkgconfig)
     return g
  def oname(self, source):
     base = os.path.splitext(source)[0]
     return '%s.o' % (base)
  def lib(self, *names):
     self.ldflags += ['-l'+ i for i in names]
  def incpath(self, *names):
     self.cflags += ['-I'+i for i in names]
  def libpath(self, *names):
     self.ldflags += ['-L'+i for i in names]
  def define(self,name, value=''):
     if value:
       name += '='+value
     self.cflags += ['-D' + name]
  def debug(self):
     self.cflags += ['-g', '-Werror']
  def optimize(self, level=2):
     self.cflags += ['-O%d'%level]
  def profile(self):
     self.cflags += ['-pg']
     self.ldflags += ['-pg']
  def cxx(self, target, source, flags=[]):
     cflags = flags + self.cflags
     target = self.oname(target)
     return (self.cc, '-c', source, '-o', self.oname(target), cflags)
  def ld(self, target, sources, flags=[]):
     ldflags = flags + self.ldflags
     objects = [self.oname(s) for s in sources]
     return (self.cc, objects, '-o', target, ldflags)
  def build(self, target, sources, flags=[]):
     flags = [] + flags + self.ldflags + self.cflags
     return (self.cc, sources, '-o', target, flags)

def compiler(**kwargs):
    return Gcc(**kwargs)


def inBuildDir(source, builddir):
    d = builddir+'/'+os.path.basename(source)
    return d

def mkdir(dir):
    if not os.access(dir, os.F_OK):
        os.makedirs(dir)

def Check(b, lib=None, header='', function='', name=''):
    t = b.copy()
    name = name or lib or function or header.replace('/','_')
    testdir = b.builddir + '/conftests/'
    testname = testdir+'test'+name
    s = testname+'.c'
    mkdir(testdir)
    f = open(s, 'wt')
    if header:
       f.write('#include "%s"\n' % header)
    if function:
       f.write('#ifdef __cplusplus\nextern "C"\n#endif\nchar %s();\n'%function)
       f.write('int main()\n{\n%s();\nreturn 0;\n}\n\n'%function)
    else:
       f.write('int main()\n{\nreturn 0;\n}\n\n')
    f.close()
    if lib:
        t.lib(lib)
    try:
       run(*t.build(testname, [s]))
    except ExecutionError as e:
       return False
    return True

def cmdline2list(s):
    r"""Split a string into words and remove quotation marks and backslashes,
as a Bourne shell would.

If e.g. pkg-config --cflags lua5.1 outputs:
-DDEB_HOST_MULTIARCH=\"x86_64-linux-gnu\" -I/usr/include/lua5.1  
this function can convert it to a list of words:
('-DDEB_HOST_MULTIARCH="x86_64-linux-gnu"', '-I/usr/include/lua5.1')
that you can then pass to subprocess.list2cmdline and get the
backslashes back.

This function neither parses nor executes any shell expansions.  For
example, it does not support the ${variable} or `command` syntaxes.
If it sees any of those, it warns and passes the metacharacters
through unchanged."""
    words = ()
    pos = 0
    # Read words from the string until we get to the end.
    while pos < len(s):
        if s[pos].isspace():
            # Spaces delimit words.  Discard them.
            pos += 1
        else:
            # Any non-space character begins a word.
            word = ""
            if pos < len(s) and s[pos] == '~':
                # An unquoted tilde at the beginning of a word should
                # refer to a home directory or to a directory stack.
                # We support neither.
                warn("tilde expansion not supported")
            while pos < len(s) and not s[pos].isspace():
                if s[pos] == '\\':
                    # An unquoted backslash escapes the following character.
                    pos += 1
                    if pos >= len(s):
                        warn("trailing backslash")
                    else:
                        word += s[pos]
                        pos += 1
                    continue
                if s[pos] == '"':
                    # A double-quoted string.  Read characters until we get
                    # to the closing double-quotation mark.
                    pos += 1
                    while True:
                        if pos >= len(s):
                            warn("unterminated double-quoted string")
                            break
                        elif s[pos] == '"':
                            # This closes the string.
                            pos += 1
                            break
                        elif s[pos] == '\\':
                            # Within a double-quoted string, a
                            # backslash followed by another backslash,
                            # double-quotation mark, dollar sign, or
                            # backtick escapes that character.
                            # Otherwise, it's just part of the string.
                            if pos + 1 >= len(s):
                                warn("trailing backslash in double-quoted string")
                            elif s[pos + 1] in '\\"$`':
                                word += s[pos + 1]
                                pos += 2
                                continue
                        elif s[pos] in "$`":
                            # Within a double-quoted string, a dollar
                            # sign or a backtick would begin some
                            # shell-expansion syntax that this
                            # function does not support.
                            warn("unsupported character %s in double-quoted string" % s[pos])
                        # Otherwise, the character is part of the string.
                        word += s[pos]
                        pos += 1
                    continue
                elif s[pos] == "'":
                    # A single-quoted string.  Backslashes have
                    # no special meaning here.  Just search for
                    # the closing single-quotation mark.
                    pos += 1
                    while True:
                        if pos >= len(s):
                            warn("unterminated single-quoted string")
                            break
                        elif s[pos] == "'":
                            # This closes the string.
                            pos += 1
                            break
                        # Otherwise, the character is part of the string.
                        word += s[pos]
                        pos += 1
                    continue
                elif s[pos] in "#$&()*;<>?[]`|":
                    # When not part of a {double,single}-quoted
                    # string, these characters invoke various shell
                    # features that this function does not support.
                    warn("unsupported character %s" % s[pos])
                word += s[pos]
                pos += 1
            # Finished reading the word; reached a space or the end of
            # the input string.
            words += (word,)
    return words

def pkgconfig(b, package):
    try:
        b.cflags += cmdline2list(shell('pkg-config', '--cflags', package).decode())
        b.ldflags += cmdline2list(shell('pkg-config', '--libs', package).decode())
    except ExecutionError as e:
        return False
    return True

def CheckLib(b, lib, header=''):
    if Check(b, lib, header):
       b.lib(lib)
       return True
    return False

def RequireLib(b, lib, header=''):
    r = CheckLib(b, lib, header)
    if not r:
       print(
         'Did not find the required %s lib or headers, exiting!' % lib)
       sys.exit(1)

def CheckLibAlternatives(b, libs, header=''):
    for i in libs:
       if CheckLib(b, i, header):
         return True

def detectLua(b):
    # lua5.1-c++ is what Debian calls a Lua library built to use C++
    # exceptions rather than longjmp.  http://bugs.debian.org/560139
    # Prefer that because it will call our C++ destructors on unwind.
    # This avoids memory leaks that would be very cumbersome to fix in
    # any other way.
    libs = 'lua5.1-c++ lua5.1 lua51 lua'.split()
    if b.usepkgconfig:
        for i in libs:
            if pkgconfig(b, i):
                return
    if CheckLibAlternatives(b, libs, header='lua.h'):
        return
    print('Did not find the Lua library, exiting !')
    sys.exit(1)

def detectOpenGl(b):
    libs = 'GL opengl3 opengl32'.split()
    if sys.platform == 'darwin':
        b.incpath('/System/Library/Frameworks/OpenGL.framework/Libraries/')
    r = CheckLibAlternatives(b, libs)
    if not r:
       print('Did not find the OpenGL library, exiting !')
       sys.exit(1)

def detectSdl(b):
    if b.usepkgconfig and pkgconfig(b, 'sdl'):
        return
    b.incpath('/usr/include/SDL')
    header = 'SDL.h'
    if '-DUSE_WIN32' in b.cflags:
        header = ''
    if CheckLib(b, 'SDL', header=header):
        return
    print('Did not find the SDL library, exiting !')
    sys.exit(1)

def detectAlwaysDynamic(b):
    RequireLib(b, 'z', 'zlib.h')
    detectOpenGl(b)
    detectSdl(b)
    if Check(b, function='strcasestr'):
       b.define('HAVE_STRCASESTR')
    if Check(b, function='strnlen'):
       b.define('HAVE_STRNLEN')
    if (Check(b, header='X11/Xlib.h') and 
        Check(b, header='X11/Xatom.h') and 
        CheckLib(b, 'X11')):
       b.define('HAVE_X')
    for i in incpaths:
       b.incpath(i)

def detectEmbedable(b):
    detectLua(b)
    RequireLib(b, 'png', 'png.h')
    if CheckLib(b, 'vorbis'):
       b.define('USE_VORBIS')
    if CheckLib(b, 'theora'):
       b.define('USE_THEORA')
    if CheckLib(b, 'ogg'):
       b.define('USE_OGG')

def detect(b):
    detectAlwaysDynamic(b)
    detectEmbedable(b)

def parallel_run(commands, jobs=2, builder=default_builder):
        """ The different commands must be independent. """
        requests, results = queue.Queue(), queue.Queue()
        for i in commands:
            c = builder.prepare(i)
            if builder.should_run(*c):
                requests.put(c)
        totalrequests = requests.qsize()
        requests.active = True

        def runQueuedCommands(requests, results):
            while requests.active:
                c = requests.get()
                if c is None:
                    return
                arglist, command = c
                builder.echo_command(command)
                try:
                     deps, outputs = builder.runner(*arglist)
                     results.put((command, deps, outputs))
                except ExecutionError as e:
                     results.put(e)
                     requests.active = True  # abort future requests

        for i in range(jobs):
            t = threading.Thread(target=runQueuedCommands, args=(requests, results))
            t.daemon = True
            requests.put(None)
            t.start()
        for _ in range(totalrequests):
           r = results.get()
           if isinstance(r, ExecutionError): raise r
           command, deps, outputs = r
           builder.store_deps(command, deps, outputs)

def runall(commands, jobs=None):
    jobs= jobs or int(main.options.jobs)
    if jobs > 1:
        parallel_run(commands)
    else:
        for i in commands:
           run(*i)

def compile(b):
    commands = [b.cxx(inBuildDir(s, b.builddir), s) for s in sources]
    runall(commands)

def link(b):
    objects = [b.oname(inBuildDir(s, b.builddir)) for s in sources]
    apptarget = b.builddir + '/' + target
    run(*b.ld(apptarget, objects))

def make(b):
    compile(b)
    link(b)

def release(builddir='fbuild/release',**kwargs):
    b = compiler(builddir=builddir, **kwargs)
    mkdir(builddir)
    detect(b)
    b.optimize()
    make(b)

def debug(builddir='fbuild/debug',**kwargs):
    b = compiler(builddir=builddir, **kwargs)
    mkdir(builddir)
    detect(b)
    b.debug()
    b.define('DEBUG')
    make(b)

def profile(builddir='fbuild/profile',**kwargs):
    b = compiler(builddir=builddir, **kwargs)
    mkdir(builddir)
    detect(b)
    b.profile()
    make(b)

class StaticGcc(Gcc):
    def __init__(self,*args,**kwargs):
        super(StaticGcc,self).__init__(*args,**kwargs)
    def lib(self, *names):
        self.ldflags += ['-Wl,-Bstatic']
        super(StaticGcc, self).lib(*names)
        self.ldflags += ['-Wl,-Bdynamic']
    def  copy(self):
        return StaticGcc(self.cflags, self.ldflags, self.cc, self.builddir, self.usepkgconfig)

def static(builddir='fbuild/static', **kwargs):
    b = compiler(builddir=builddir, usepkgconfig=False, **kwargs)
    b.incpath('engine/apbuild')
    b.incpath('deps/incs')
    b.libpath('deps/libs')
    b.cflags += [
        '-fno-stack-protector', # disable stack protection to avoid dependency on
                                                #__stack_chk_fail@@GLIBC_2.4
        '-U_FORTIFY_SOURCE', # requires glibc 2.3.4 or higher
        '-include', 'engine/apbuild/apsymbols.h',
       ]
    b.ldflags += [
        '-Wl,--as-needed',  # must be set after all object files or the binary breaks
        '-Wl,--hash-style=both', # By default FC6 only generates a .gnu.hash section
                              # Do all main distros support .gnu.hash now ?
        '-static-libgcc',
        '-Wl,-Bstatic', '-lstdc++', '-Wl,-Bdynamic',
        '-Wl,-s'
        ]
    p = os.popen(b.cc + ' -print-file-name=libstdc++.a')
    stdcxx = p.read().strip()
    run('ln', '-sf', stdcxx)
    
    mkdir(builddir)
    detectAlwaysDynamic(b)
    b = StaticGcc(b.cflags,b.ldflags, b.cc,b.builddir, usepkgconfig=False)
    detectEmbedable(b)
    b.optimize()
    b.libpath('.')
    make(b)

def mingw(builddir='fbuild/mingw', cc='i486-mingw32-g++', **kwargs):
    b = compiler(builddir=builddir, cc=cc, usepkgconfig=False, **kwargs)
    b.define('USE_WIN32')
    b.incpath('mingwdeps/include')
    b.libpath('mingwdeps/lib')
    b.lib('mingw32', 'SDLmain', 'wsock32', 'ws2_32')
    b.ldflags += ['-mwindows']
    mkdir(builddir)
    detect(b)
    b.cflags += ['-UHAVE_STRCASESTR','-UHAVE_STRNLEN']
    make(b)

def clean():
    autoclean()

# Keep this equivalent to languages/genpot.sh.
def pot():
    luas = find('.', '*.lua')
    luas.sort()
    run('xgettext','-d','bos','-k_','-o','languages/bos.pot', luas)
    s = sorted(sources)
    run('xgettext','-d','engine','-C','-k_','--add-comments=TRANSLATORS','-o','languages/engine.pot', s)

def all(**kwargs):
    release(**kwargs)
    debug(**kwargs)

def default(**kwargs):
    release(**kwargs)

setup(default='default')
if __name__ == '__main__':
    import optparse
    j = optparse.Option('-j', '--jobs', action='store',
                      help='the number of jobs to run simultaneously',
                      default=1)
    main(extra_options=[j])




#! /usr/bin/env python
#     ____                _       __               
#    / __ )____  _____   | |     / /___ ___________
#   / __  / __ \/ ___/   | | /| / / __ `/ ___/ ___/
#  / /_/ / /_/ (__  )    | |/ |/ / /_/ / /  (__  ) 
# /_____/\____/____/     |__/|__/\__,_/_/  /____/  
#                                              
#       A futuristic real-time strategy game.
#          This file is part of Bos Wars.
#
#      Script that generates all the distribution packages.
#      (c) Copyright 2007 by Francois Beerten
#
#      Bos Wars is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published
#      by the Free Software Foundation; only version 2 of the License.
#
#      Bos Wars is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
import os
import sys
import csv
import re

importantkeys = ['Name', 
                 'EnergyValue', 'MagmaValue',
                 'MaxEnergyUtilizationRate', 'MaxMagmaUtilizationRate',
                 'EnergyProductionRate', 'MagmaProductionRate',
                 'EnergyStorageCapacity', 'MagmaStorageCapacity',
                 'HitPoints', 'SightRange', 'Armor', 'BasicDamage', 
                 'PiercingDamage', 'MaxAttackRange', 'MovementDelay',
                 'AttackDelay']

def findunits():
    u = os.listdir('units')
    u = [x for x in u if not x.startswith('.')]
    return u

def findunitscripts(directory):
    base = 'units/' + directory + '/'
    s = os.listdir(base)
    s = [base + x for x in s if x.endswith('.lua')]
    return s

def findallscripts():
    scripts = []
    units = findunits()
    for u in units:
        scripts.extend(findunitscripts(u))
    return scripts

def parseKvList(kvlist):
    d = {}
    key = kvlist[0]
    for i in kvlist[1]:
        if key:
            d[key] = i
            key = None
        else:
            key = i
    return key

def replaceTabs(s):
    return s.replace('\t', ' ' * 4)

class ParsedScript:
    def __init__(self, path):
        self.path = path
        self.units = []
    def regenerate(self, f):
        f.write(self.head)
        for unit in self.units:
            unit.regenerate(f)
    
class ParsedUnit:
    def __init__(self):
        self.orderedkeys = []
        self.stats = {}
    def regenerateStats(self, f):
        keys = self.stats.keys()
        keys.sort()
        for k in self.orderedkeys[:-1]:
            f.write('    ' + k + ' = ' + self.stats[k] + ',\n')
        k = self.orderedkeys[-1]
        f.write('    ' + k + ' = ' + self.stats[k] + '\n')
    def regenerate(self, f):
        f.write('DefineUnitType("' + self.internalname + '", {\n')
        self.regenerateStats(f)
        f.write('})')
        f.write(self.rest)
    def writeCsv(self, statsfile):
        statsfile.writerow(self.stats) 
        

def stripComments(s):
    s = s.split('\n')
    new = []
    for i in s:
        i = i.split('--', 1)
        new.append(i[0])
    return '\n'.join(new)

def parseDefineUnitType(text):
    unit = ParsedUnit()
    body, rest = text.split('})', 1)
    unit.rest = replaceTabs(rest)
    body = stripComments(body)
    internalname, stats = body.split(', {', 1)
    unit.internalname = internalname.strip().strip('"')
    stats = stats.split('=')
    prevKey = stats[0].strip()
    for x in stats[1:-1]:
        x = x.split(',')
        unit.stats[prevKey] = replaceTabs(','.join(x[:-1]).strip())
        unit.orderedkeys.append(prevKey)
        prevKey = x[-1].strip()
    if len(stats) > 2:
        unit.orderedkeys.append(prevKey)
        unit.stats[prevKey] = replaceTabs(stats[-1].strip())
    return unit

def parseWaits(text, section):
    section += ' ='
    if section not in text:
        return 0
    _, text = text.split(section, 1)
    text,_ = text.split('}',1)
    if 'unbreakable' in text:
        _, text, _ = text.split('"unbreakable', 2)
    m =re.findall(r'"wait \d+"', text)
    waits = 0
    for i in m:
        v = int(i[6:-1])
        waits += v
    return waits

def parseScript(path):
    parsedscript = ParsedScript(path)
    s = file(path, 'rt').read()
    elements = s.split('DefineUnitType(')
    parsedscript.head = elements[0]
    movement = parseWaits(s, 'Move')
    attackwait = parseWaits(s, 'Attack')
    for e in elements[1:]:
        unit = parseDefineUnitType(e)
        unit.stats['MovementDelay'] = movement
        unit.stats['AttackDelay'] = attackwait
        parsedscript.units.append(unit)
    return parsedscript


def parseAllScripts():
    units = []
    scriptpaths = findallscripts()
    scripts = []
    for i in scriptpaths:
        if 'crystal' not in i:
            parsedscript = parseScript(i)
            scripts.append(parsedscript)
            units.extend(parsedscript.units)
    return units, scripts

def generateStatsFile(units):
    rawcsvfile = file('unitstats.csv', 'wb')
    statsfile = csv.DictWriter(rawcsvfile, importantkeys, extrasaction='ignore',
                               delimiter=';', quotechar="'")
    title = {}
    for i in importantkeys:
        title[i]=i
    statsfile.writerow(title)
    for unit in units:
        unit.writeCsv(statsfile)


def regenerateScripts(scripts):
    for i in scripts:
        f = file(i.path, 'wt')
        i.regenerate(f)

def readUnitStats():
    rawcsvfile = file('unitstats.csv', 'rb')
    stats = csv.DictReader(rawcsvfile, delimiter=';', quotechar="'")
    newstats = {}
    for r in stats:
        newstats['"%s"' % r['Name']] = r
    return newstats
def removeCommas(old):
    return ''.join(old.split(','))
def updateUnitStats(units):
    stats = readUnitStats()
    for unit in units:
        name = unit.stats['Name']
        if stats.has_key(name):
            up = stats[name]
            for k in up.keys():
                if unit.stats.has_key(k) and k != 'Name':
                    unit.stats[k] = removeCommas(up[k])

Usage = """
    Unit stats generation tool.
    Usage: %s <command>

    Command:
        csv
        regenerate
        update
    
    When updating, the unitstats.csv file should use the semicolon (;) as
    delimiter and single quote (') as string quote.
"""
def printUsage(args):
    print Usage % args[0]

def main(args):
    if len(args) == 1:
        printUsage(args)
        return
    units, scripts = parseAllScripts()
    if args[1] == 'csv':
        generateStatsFile(units)
    elif args[1] == 'regenerate':
        regenerateScripts(scripts)
    elif args[1] == 'update':
        updateUnitStats(units)
        regenerateScripts(scripts)
    elif args[1] == 'tupdate':
        updateUnitStats([units[0]])
        regenerateScripts(scripts)
    else:
        printUsage(args)
        return


if __name__ == '__main__':
    main(sys.argv)
    



#! /usr/bin/env python

##     ____                _       __               
##    / __ )____  _____   | |     / /___ ___________
##   / __  / __ \/ ___/   | | /| / / __ `/ ___/ ___/
##  / /_/ / /_/ (__  )    | |/ |/ / /_/ / /  (__  ) 
## /_____/\____/____/     |__/|__/\__,_/_/  /____/  
##                                              
##       A futuristic real-time strategy game.
##          This file is part of Bos Wars.
##
##      Script that generates all the distribution packages.
##      (c) Copyright 2007 by Francois Beerten
##
##      Stratagus is free software; you can redistribute it and/or modify
##      it under the terms of the GNU General Public License as published
##      by the Free Software Foundation; only version 2 of the License.
##
##      Stratagus is distributed in the hope that it will be useful,
##      but WITHOUT ANY WARRANTY; without even the implied warranty of
##      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##      GNU General Public License for more details.

import sys
import os
import tarfile
import zipfile
import time

def printDot():
  sys.stdout.write('.')
  sys.stdout.flush()

def createTarArchive(archivename, namelist):
  print "\nCreating %s.tar.gz" % archivename
  tar = tarfile.open(archivename + ".tar.gz", "w:gz")
  for name in namelist:
      tarinfo = tar.gettarinfo(name, archivename + "/" + name)
      tarinfo.uid = 500
      tarinfo.gid = 500
      tarinfo.uname = "user"
      tarinfo.gname = "user"
      printDot()
      tar.addfile(tarinfo, file(name))
  tar.close()
  print

def createZipArchive(archivename, namelist):
  print "\nCreating %s.zip" % archivename
  z = zipfile.ZipFile(archivename + ".zip", "w", zipfile.ZIP_DEFLATED)
  for name in namelist:
      z.write(name, archivename + "/" + name)
      printDot()
  z.close()
  print


def isSvnWorkSpace():
   return os.access('.svn', os.F_OK)

def listSvnFiles():
   entries = os.popen('svn ls -R').readlines()
   files = []
   for e in entries:
     e = e.strip()
     if not e.endswith('/'):
        files.append(e)
   return files

def listFilesInDirectories():
   raise "Not implemented"

def listFiles():
   if isSvnWorkSpace():
      return listSvnFiles()
   else:
      return listFilesInDirectories()

def buildDefaultVersionName():
   t = time.time()
   v = time.strftime('%Y%m%d', time.gmtime(t))
   return v

def buildVersionName():
   if len(sys.argv) > 1:
      return sys.argv[1]
   else:
      return buildDefaultVersionName()

def filterDirectories(files):
   return [x for x in files if not x.endswith('/')]
def filterSources(files):
   buildfiles = ['SConstruct', 'bos.sln', 'make.py', 'fabricate.py']
   def isGoodFile(f):
     return (not f.startswith('engine/') and not f.startswith('tools/') and
             f not in buildfiles)
   return [x for x in files if isGoodFile(x)]
   


def buildPackages():
  name = 'boswars-' + buildVersionName() + '-%s'

  os.chdir('../..')
  files = listFiles()
  files = filterDirectories(files)

  # create source packages
  createTarArchive(name % 'src', files)
  createZipArchive(name % 'src', files)

  # create binary packages
  #os.popen('scons static=1 debug=0 CXX=apg++')
  binarydistfiles = filterSources(files)
  linuxfiles = binarydistfiles[:]
  linuxfiles.append('boswars')
  createTarArchive(name % 'linux', linuxfiles)
  if os.access('boswars.exe', os.F_OK):
     winfiles = binarydistfiles[:]
     winfiles.append('boswars.exe')
     createZipArchive(name % 'win32', winfiles)
  

if __name__ == '__main__':
  buildPackages()








#  Pretty printing of SDL objects in GDB.
#
#  (c) Copyright 2011 by Kalle Olavi Niemitalo.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
Pretty printing of SDL objects in the GNU Debugger.
This requires GDB 7.0 or later, configured --with-python.
The code was written for Python 2.6.6 but may work with earlier versions too.
"""

import gdb

class Uint8Printer:
    "Pretty-print a Uint8."

    def __init__(self, val):
        object.__init__(self)
        self.val = val

    def to_string(self):
        "Called by GDB to find what to display as the value."
        # Stop GDB from displaying it as a character literal.
        return self.val.cast(gdb.lookup_type("unsigned int"))

def lookup_printer(val):
    "Return a printer for the value, if supported; else None."
    type = val.type
    while type.code == gdb.TYPE_CODE_TYPEDEF:
        tag = str(type)
        if tag == "Uint8":
            return Uint8Printer(val)
        type = type.target()

def register_printers(objfile):
    """Register all the pretty printers defined by this module.
OBJFILE may be an objfile, a progspace, or the entire gdb module."""
    objfile.pretty_printers.append(lookup_printer)



#  Pretty printing of C++ STL objects in GDB.
#
#  (c) Copyright 2010 by Kalle Olavi Niemitalo.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
Pretty printing of C++ STL objects in the GNU Debugger.
This requires GDB 7.0 or later, configured --with-python.
The code was written for Python 2.5.5 but may work with earlier versions too.
The C++ STL pretty printers support libstdc++ 6.0.13 of GCC 4.4.4.

These pretty printers should not be used with GCC 4.5 or later,
because those versions of libstdc++ already include pretty printers
maintained as part of the library and therefore better up to date.
According to the documentation, GDB looks for pretty printers first
in the objfiles, then in the progspace, and last in the gdb module;
thus, if you register these pretty printers in the gdb module and
let GDB register the pretty printers of libstdc++ in the objfile of
libstdc++, then they should automatically take precedence over these.
"""

import gdb
import re

class StdStringPrinter:
    "Pretty-print an std::string."

    __typeRegex = re.compile("^std::basic_string<.*>$")

    def __init__(self, val):
        object.__init__(self)
        self.val = val

    def display_hint(self):
        "Called by GDB to find how to display the value and children."
        return "string"

    def to_string(self):
        "Called by GDB to find what to display as the value."
        data = self.val["_M_dataplus"]["_M_p"]
        if data == 0:
            return data
        repType = gdb.lookup_type(str(self.val.type.strip_typedefs().unqualified()) + "::_Rep")
        rep = data.cast(repType.pointer()) - 1
        return data.string(length = rep["_M_length"])

    @staticmethod
    def lookup_printer(val):
        "Return a printer for the value, if supported; else None."
        tag = val.type.strip_typedefs().tag
        if tag and StdStringPrinter.__typeRegex.match(tag):
            return StdStringPrinter(val)

class StdVectorPrinter:
    "Pretty-print an std::vector."

    __typeRegex = re.compile("^std::vector<.*>$")

    def __init__(self, val):
        object.__init__(self)
        self.val = val

    def children(self):
        "Called by GDB to enumerate the children of the value."
        start = self.val["_M_impl"]["_M_start"]
        finish = self.val["_M_impl"]["_M_finish"]
        for index in range(finish - start):
            element = (start + index).dereference()
            yield "[%d]" % index, element

    def display_hint(self):
        "Called by GDB to find how to display the value and children."
        return "array"

    def to_string(self):
        "Called by GDB to find what to display as the value."
        return "std::vector"

    @staticmethod
    def lookup_printer(val):
        "Return a printer for the value, if supported; else None."
        tag = val.type.tag
        if tag and StdVectorPrinter.__typeRegex.match(tag):
            return StdVectorPrinter(val)

class StdListPrinter:
    "Pretty-print an std::list."

    __typeRegex = re.compile("^std::list<.*>$")

    def __init__(self, val):
        object.__init__(self)
        self.val = val

    def children(self):
        "Called by GDB to enumerate the children of the value."
        elementType = self.val.type.template_argument(0)
        nodeTypeName = "std::_List_node<%s >" % elementType
        nodeType = gdb.lookup_type(nodeTypeName)
        nodePointerType = nodeType.pointer()
        sentinelPointer = self.val["_M_impl"]["_M_node"].address
        nodeBasePointer = sentinelPointer
        while nodeBasePointer.dereference()["_M_next"] != sentinelPointer:
            nodeBasePointer = nodeBasePointer.dereference()["_M_next"]
            nodePointer = nodeBasePointer.cast(nodePointerType)
            yield "element", nodePointer.dereference()["_M_data"]

    def display_hint(self):
        "Called by GDB to find how to display the value and children."
        return "array"

    def to_string(self):
        "Called by GDB to find what to display as the value."
        return "std::list"

    @staticmethod
    def lookup_printer(val):
        "Return a printer for the value, if supported; else None."
        tag = val.type.tag
        if tag and StdListPrinter.__typeRegex.match(tag):
            return StdListPrinter(val)

class StdMapPrinter:
    "Pretty-print an std::map."

    __typeRegex = re.compile("^std::map<.*>$")

    def __init__(self, val):
        object.__init__(self)
        self.val = val

    def children(self):
        "Called by GDB to enumerate the children of the value."
        keyType = self.val.type.template_argument(0)
        valueType = self.val.type.template_argument(1)
        nodeTypeName = "std::_Rb_tree_node<std::pair<%s const, %s > >" \
            % (keyType, valueType)
        nodeType = gdb.lookup_type(nodeTypeName)
        nodePointerType = nodeType.pointer()
        def recurse(nodeBasePointer):
            if nodeBasePointer != 0:
                nodeBase = nodeBasePointer.dereference()
                node = nodeBasePointer.cast(nodePointerType).dereference()
                for tag, data in recurse(nodeBase["_M_left"]):
                    yield tag, data
                yield "key", node["_M_value_field"]["first"]
                yield "value", node["_M_value_field"]["second"]
                for tag, data in recurse(nodeBase["_M_right"]):
                    yield tag, data
        return recurse(self.val["_M_t"]["_M_impl"]["_M_header"]["_M_parent"])

    def display_hint(self):
        "Called by GDB to find how to display the value and children."
        return "map"

    def to_string(self):
        "Called by GDB to find what to display as the value."
        return "std::map"

    @staticmethod
    def lookup_printer(val):
        "Return a printer for the value, if supported; else None."
        tag = val.type.tag
        if tag and StdMapPrinter.__typeRegex.match(tag):
            return StdMapPrinter(val)

def register_printers(objfile):
    """Register all the pretty printers defined by this module.
OBJFILE may be an objfile, a progspace, or the entire gdb module."""
    objfile.pretty_printers.append(StdStringPrinter.lookup_printer)
    objfile.pretty_printers.append(StdVectorPrinter.lookup_printer)
    objfile.pretty_printers.append(StdListPrinter.lookup_printer)
    objfile.pretty_printers.append(StdMapPrinter.lookup_printer)



#  Pretty printing of Bos Wars objects in GDB.
#
#  (c) Copyright 2010 by Kalle Olavi Niemitalo.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""
Pretty printing of Bos Wars objects in the GNU Debugger.
This requires GDB 7.0 or later, configured --with-python.
The code was written for Python 2.5.5 but may work with earlier versions too.
"""

import gdb

class COrderPrinter:
    "Pretty-print a COrder."

    def __init__(self, val):
        object.__init__(self)
        self.val = val

    def children(self):
        "Called by GDB to enumerate the children of the value."
        for field in self.val.type.fields():
            if field.name == "Action":
                # COrder::Action is defined as unsigned char, but its
                # possible values are listed in enum _unit_action_.
                yield field.name, self.val[field.name].cast(gdb.lookup_type("enum _unit_action_"))
            elif field.name == "Width" or field.name == "Height":
                # These are unsigned char but cast them to int so
                # gdb won't try to display them as e.g. '\0'.
                yield field.name, self.val[field.name].cast(gdb.lookup_type("int"))
            elif field.name == "Arg1":
                # COrder::Action shows which member of the union is used.
                for enum in gdb.lookup_type("enum _unit_action_").fields():
                    if enum.bitpos == self.val["Action"]:
                        if enum.name == "UnitActionPatrol":
                            yield "Arg1.Patrol", self.val["Arg1"]["Patrol"];
                        elif enum.name == "UnitActionSpellCast":
                            yield "Arg1.Spell", self.val["Arg1"]["Spell"];
            else:
                yield field.name, self.val[field.name]

    def to_string(self):
        "Called by GDB to find what to display as the value."
        return "COrder"

class CUnitTypePrinter:
    "Pretty-print a CUnitType."

    def __init__(self, val):
        object.__init__(self)
        self.val = val

    def children(self):
        "Called by GDB to enumerate the children of the value."
        for field in self.val.type.fields():
            if field.name == "FieldFlags" or field.name == "MovementMask":
                # These are flags, so print in hexadecimal.
                yield field.name, "%#x" % self.val[field.name]
            else:
                yield field.name, self.val[field.name]

    def to_string(self):
        "Called by GDB to find what to display as the value."
        return "CUnitType"

class COrderPointerPrinter:
    "Pretty-print a pointer to COrder."

    def __init__(self, val):
        object.__init__(self)
        self.val = val

    def to_string(self):
        "Called by GDB to find what to display as the value."
        voidPointerType = gdb.lookup_type("void").pointer()
        # str(self.val) would cause a recursive call; the cast avoids that.
        result = str(self.val.cast(voidPointerType))
        if self.val != 0:
            order = self.val.dereference()
            action = order["Action"].cast(gdb.lookup_type("enum _unit_action_"))
            result = result + (" {%s}" % action)
        return result

class CPatchPointerPrinter:
    "Pretty-print a pointer to CPatch."

    def __init__(self, val):
        object.__init__(self)
        self.val = val

    def to_string(self):
        "Called by GDB to find what to display as the value."
        voidPointerType = gdb.lookup_type("void").pointer()
        # str(self.val) would cause a recursive call; the cast avoids that.
        result = str(self.val.cast(voidPointerType))
        if self.val != 0:
            patch = self.val.dereference()
            result = result + (" {(%d,%d)" % (patch["x"], patch["y"]))
            patchTypePointer = patch["type"]
            if patchTypePointer != 0:
                patchType = patchTypePointer.dereference()
                result = result + (" %s" % patchType["name"])
            result = result + "}"
        return result

class StructPointerPrinter:
    "Pretty-print a pointer to a structure that has one important member."

    def __init__(self, val, memberName, indirectMemberName = None):
        object.__init__(self)
        self.val = val
        self.memberName = memberName
        self.indirectMemberName = indirectMemberName

    def to_string(self):
        "Called by GDB to find what to display as the value."
        voidPointerType = gdb.lookup_type("void").pointer()
        # str(self.val) would cause a recursive call; the cast avoids that.
        result = str(self.val.cast(voidPointerType))
        if self.val != 0:
            memberValue = self.val.dereference()[self.memberName]
            if self.indirectMemberName == None:
                result = result + (" {%s}" % memberValue)
            elif memberValue != 0:
                memberValue = memberValue.dereference()[self.indirectMemberName]
                result = result + (" {%s}" % memberValue)
        return result

def lookup_printer(val):
    "Return a printer for the value, if supported; else None."
    type = val.type
    if type.code == gdb.TYPE_CODE_STRUCT:
        tag = type.tag      # might be None
        if tag == "COrder":
            return COrderPrinter(val)
        elif tag == "CUnitType":
            return CUnitTypePrinter(val)
    elif type.code == gdb.TYPE_CODE_PTR:
        tag = type.target().tag # might be None
        if tag == "CConstruction":
            return StructPointerPrinter(val, "Ident")
        elif tag == "CGraphic":
            return StructPointerPrinter(val, "File")
        elif tag == "CIcon":
            return StructPointerPrinter(val, "Ident")
        elif tag == "COrder":
            return COrderPointerPrinter(val)
        elif tag == "CPatch":
            return CPatchPointerPrinter(val)
        elif tag == "CPatchType":
            return StructPointerPrinter(val, "name")
        elif tag == "CPlayerColorGraphic":
            return StructPointerPrinter(val, "File")
        elif tag == "CUnit":
            return StructPointerPrinter(val, "Type", "Ident")
        elif tag == "CUnitType":
            return StructPointerPrinter(val, "Ident")
        elif tag == "Missile":
            return StructPointerPrinter(val, "Type", "Ident")
        elif tag == "MissileType":
            return StructPointerPrinter(val, "Ident")

def register_printers(objfile):
    """Register all the pretty printers defined by this module.
OBJFILE may be an objfile, a progspace, or the entire gdb module."""
    objfile.pretty_printers.append(lookup_printer)
